/*
 * ----------------------------------------------------
 *
 * Pin Level Emulation of the M93c46 Microwire EEProm
 * required for the emulation of the STE10/100 
 * Network chip.
 *
 * (C) 2004  Lightmaze Solutions AG
 *   Author: Jochen Karrer
 *
 * State: Working, confusing, Write-Enable command is missing 
 *	  not yet using the new Signal level emulation scheme
 *	  not yet written to a file
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 * ----------------------------------------------------
 */

#include <stdint.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <m93c46.h>

#if 0
#define dprintf(x...) { fprintf(stderr,x); }
#else
#define dprintf(x...)
#endif

#define STATE_IDLE 	      (0)
#define STATE_WAIT_FOR_START  (3)
#define STATE_COMMAND         (1)
#define STATE_RD_ADDRESS      (4)
#define STATE_WR_ADDRESS      (5)
#define STATE_READ            (6)
#define STATE_WRITE           (7)
#define STATE_IGNORE	      (8)
#define STATE_ACK1	      (9)
#define STATE_ACK2	      (10)

#define  CMD_READ	(2)
#define  CMD_WRITE	(1)

#define BP_START (0)
#define BP_CLK (1)
#define BP_DONE	 (2)

static void 
print_state(uint8_t pinstate,int state) {
	if(pinstate&MW_CS) {
		dprintf("S");
	} else {
		dprintf("s");
	}
	if(pinstate&MW_SCLK) {
		dprintf("C");
	} else {
		dprintf("c");
	}
	if(pinstate&MW_SDI) {
		dprintf("I");
	} else {
		dprintf("i");
	}
	if(pinstate&MW_SDO) {
		dprintf("O");
	} else {
		dprintf("o");
	}
	dprintf(" - ");
	switch(state) {
		case STATE_IDLE: 
			dprintf("IDLE ");
			break;
		case STATE_WAIT_FOR_START: 
			dprintf("WAIT ");
			break;
		case STATE_COMMAND: 
			dprintf("CMD  ");
			break;
		case STATE_RD_ADDRESS:
			dprintf("RADDR");
			break;
		case STATE_WR_ADDRESS:
			dprintf("WADDR");
			break;
		case STATE_READ: 
			dprintf("READ ");
			break;
		case STATE_WRITE:
			dprintf("WRITE");
			break;
		case STATE_IGNORE:
			dprintf("IGN  ");
			break;
		case STATE_ACK1:
			dprintf("ACK1 ");
			break;
		case STATE_ACK2:
			dprintf("ACK2 ");
			break;
	}
}
static int
receive_bit(M93C46 *eprom,uint8_t pinstate) {
	dprintf(" bitphase %d ",eprom->bitphase);
	switch(eprom->bitphase) {
		case BP_START:
			if(pinstate&MW_SCLK) {
				eprom->bitphase=BP_CLK;
				if(pinstate&MW_SDI) {
					eprom->inbuf=(eprom->inbuf<<1) |1;
				} else {
					eprom->inbuf<<=1;
				}
				dprintf("\nInbuf %02x\n",eprom->inbuf);
			}
			break;
		case BP_CLK:
			if(!(pinstate&MW_SCLK)) {
				eprom->bitphase=BP_DONE;
			}
			break;
		case BP_DONE:
			eprom->bitphase=BP_START;
			break;
	}
	dprintf("after: %d ",eprom->bitphase);
	return eprom->bitphase;
}

uint8_t
m93c46_feed(M93C46 *eprom,uint8_t pinstate) {
	//dprintf("eprom state %d, pinstate %02x, ",eprom->state,pinstate);
	print_state(pinstate,eprom->state);
	if(!(pinstate&MW_CS) && (eprom->state!=STATE_ACK1) && (eprom->state!=STATE_ACK2)) {
		eprom->state=STATE_IDLE;		
		eprom->bitphase=BP_START;
		eprom->inbuf=0;
		return pinstate;
	}
	switch (eprom->state) {
		case STATE_IDLE:
			if(pinstate&MW_SDI) {
				dprintf("SDI is high in IDLE state\n");
			}
			eprom->bitphase=BP_START;
			eprom->inbuf=0;
			eprom->state=STATE_WAIT_FOR_START;
			break;

		case STATE_WAIT_FOR_START:
			{
				int bphase=receive_bit(eprom,pinstate);
				if(bphase==BP_DONE) {
					if(eprom->inbuf==1) {
						eprom->state=STATE_COMMAND;
						eprom->inbuf=0;
						eprom->expected_bits=2;
					} 
					eprom->bitphase=BP_START;
				} else if(bphase==BP_START) {
					pinstate|=MW_SDO;	// acknowlegde for write
				} 
			}
			break;

		case STATE_COMMAND:
			if(receive_bit(eprom,pinstate)==BP_DONE) {
				eprom->expected_bits--;
				if(eprom->expected_bits==0) {
					switch(eprom->inbuf) {
						case  CMD_READ:
							eprom->state=STATE_RD_ADDRESS;
							eprom->inbuf=0;
							eprom->expected_bits=6;
							break;
						case  CMD_WRITE:
							eprom->state=STATE_WR_ADDRESS;
							eprom->inbuf=0;
							eprom->expected_bits=6;
							break;
						default:
							eprom->state=STATE_IGNORE;
							break;
					}
				}
				eprom->bitphase=BP_START;
			}
			break;
		case STATE_RD_ADDRESS:
			if(receive_bit(eprom,pinstate)==BP_DONE) {
                                eprom->expected_bits--;
                                if(eprom->expected_bits==0) {
					eprom->address=eprom->inbuf;
					eprom->state=STATE_READ;
					eprom->inbuf=0;
					eprom->expected_bits=16;
                                }
                                eprom->bitphase=BP_START;
                        }
			break;

		case STATE_WR_ADDRESS:
			if(receive_bit(eprom,pinstate)==BP_DONE) {
                                eprom->expected_bits--;
                                if(eprom->expected_bits==0) {
					eprom->address=eprom->inbuf;
					eprom->state=STATE_WRITE;
					eprom->inbuf=0;
					eprom->expected_bits=16;
                                }
                                eprom->bitphase=BP_START;
                        }
			break;

		case STATE_READ:
			{
				 int bphase=receive_bit(eprom,pinstate);
				 if(bphase==BP_DONE) {
					eprom->expected_bits--;
					if(eprom->expected_bits==0) {
						eprom->state=STATE_IGNORE;
					}
					eprom->bitphase=BP_START;
				} else if(bphase==BP_CLK) { 
					uint16_t data=eprom->data[2*eprom->address] + (eprom->data[2*eprom->address+1]<<8);
					int bitnr=eprom->expected_bits-1;
					dprintf("addr %02x data %04x bit %d\n",eprom->address,data,bitnr);
					if(data&(1<<bitnr)) {
						pinstate|=MW_SDO;
					}
				}
			}
			break;
			
		case STATE_WRITE:
			if(receive_bit(eprom,pinstate)==BP_DONE) {
                                eprom->expected_bits--;
                                if(eprom->expected_bits==0) {
					eprom->state=STATE_ACK1;
					eprom->data[2*eprom->address]=eprom->inbuf&0xff;
					eprom->data[2*eprom->address+1]=(eprom->inbuf>>8)&0xff;
                                }
                                eprom->bitphase=BP_START;
                        }
			break;

		case STATE_ACK1:
			if(pinstate&MW_CS) {
				eprom->state=STATE_ACK2;
				pinstate|=MW_SDO;
			}
			break;
		case STATE_ACK2:
			if(!(pinstate&MW_CS)) {
				eprom->state=STATE_IDLE;
			} else {
				pinstate|=MW_SDO;
			}
			
			

		case STATE_IGNORE:
			break;
			
	}
	dprintf(" leave: ");
	print_state(pinstate,eprom->state);
	dprintf("\n");
	return pinstate;
}

/*
 * -------------------------------------------------------------------
 * This trace of the signal node is invoked whenever CLK line changes
 * -------------------------------------------------------------------
 */
static int
SCLK_Change(SigNode *node,int value,void *clientData)
{
	M93C46 *eprom = clientData;
	int result;
	if(value==SIG_HIGH) {
		eprom->pinstate |= MW_SCLK;
	} else if (value == SIG_LOW)  {
		eprom->pinstate &= ~MW_SCLK;
	}
	result = m93c46_feed(eprom,eprom->pinstate); 
	if(result & MW_SDO) {
	 	SigNode_Set(eprom->sdo,SIG_HIGH);
	} else {
	 	SigNode_Set(eprom->sdo,SIG_LOW);
	}
	return 0;
}

static int
SDI_Change(SigNode *node,int value,void *clientData)
{
	M93C46 *eprom = clientData;
	int result;
	if(value==SIG_HIGH) {
		eprom->pinstate |= MW_SDI;
	} else if (value == SIG_LOW)  {
		eprom->pinstate &= ~MW_SDI;
	}
	result = m93c46_feed(eprom,eprom->pinstate); 
	if(result & MW_SDO) {
	 	SigNode_Set(eprom->sdo,SIG_HIGH);
	} else {
	 	SigNode_Set(eprom->sdo,SIG_LOW);
	}
	return 0;
}

M93C46 *
m93c46_New(char *name) {
	int i;
	M93C46 *eprom;
	char *nodename=alloca(strlen(name)+20);
	eprom=malloc(sizeof(M93C46));
	if(!eprom) {
		fprintf(stderr,"Out of memory allocating EEPROM\n");
		exit(342);
	}
	memset(eprom,0,sizeof(M93C46));
        sprintf(nodename,"%s.sclk",name);
        eprom->sclk = SigNode_New(nodename);
        if(!eprom->sclk) {
                fprintf(stderr,"Can not create node %s for Microwire SCLK\n",nodename);
                exit(2763);
        }
	eprom->sclkTrace = SigNode_Trace(eprom->sclk,SCLK_Change,eprom);
        sprintf(nodename,"%s.sdi",name);
        eprom->sdi = SigNode_New(nodename);
        if(!eprom->sdi) {
                fprintf(stderr,"Can not create node %s for Microwire SDI\n",nodename);
                exit(2763);
        }
	eprom->sdiTrace = SigNode_Trace(eprom->sdi,SDI_Change,eprom);
        sprintf(nodename,"%s.sdo",name);
        eprom->sdo = SigNode_New(nodename);
        if(!eprom->sdo) {
                fprintf(stderr,"Can not create node %s for Microwire SDO\n",nodename);
                exit(2763);
        }
	for(i=0;i<256;i++) {
		eprom->data[i]=0xff;
	}
	fprintf(stderr,"M93C64 Microwire EEProm \"%s\" created\n",name);
	return eprom;
}
