/*
 * --------------------------------------------------------------------------
 *
 * Emulation of ARM Primecell PL190 Vectored interrupt controller 
 *
 * (C) 2005 Jochen Karrer
 *    Author: Jochen Karrer
 *
 * Status: nothing implemented
 * 	Chaining of multiple Interrupt controllers not implemented
 *
 *
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * --------------------------------------------------------------------------
 */

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include "bus.h"
#include "signode.h"

#define VICIRQSTATUS	(0)
#define VICFIQSTATUS	(4)
#define VICRAWINTR 	(8)	
#define VICINTSELECT	(0xc)
#define VICINTENABLE	(0x10)
#define VICINTENCLEAR	(0x14)
#define VICSOFTINT	(0x18)
#define VICSOFTINTCLEAR	(0x1c)
#define VICPROTECTION	(0x20)
#define VICVECTADDR	(0x30)
#define VICDEFVECTADDR  (0x34)
#define VICVECTADDRN(n) (0x100 + ((n)<<2))
#define VICVECCNTL(n)	(0x200 + ((n)<<2))
#define VICPERIPHID(n)  (0xfe4 + ((n)<<2))
#define VICPCELLID(n)	(0xff0 + ((n)<<2))

typedef struct PL190 {
	BusDevice bdev;
	uint32_t irqstatus;
	uint32_t fiqstatus;
	uint32_t rawintr;	
	uint32_t intselect;
	uint32_t intenable;
	uint32_t intenclear;
	uint32_t softint;
	uint32_t softintclear;
	uint32_t protection;
	uint32_t vectaddr;
	uint32_t defvectaddr;
	uint32_t vectaddrn[16];
	uint32_t veccntl[16];
	SigNode *intSourceNode[32];
	SigTrace *intSourceTrace[32];
	
} PL190;

static int
int_source_change(SigNode *node,int value,void *clientData)
{
	return 0;
}

uint32_t
irqstatus_read(void *clientData,uint32_t address,int rqlen) 
{
	return 0;
}

static void
irqstatus_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC register %08x is not implemented\n",address);
        return;
}
uint32_t
fiqstatus_read(void *clientData,uint32_t address,int rqlen) 
{
	return 0;
}

static void
fiqstatus_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC register %08x is not implemented\n",address);
        return;
}
uint32_t
rawintr_read(void *clientData,uint32_t address,int rqlen) 
{
	return 0;
}

static void
rawintr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC register %08x is not implemented\n",address);
        return;
}
uint32_t
intselect_read(void *clientData,uint32_t address,int rqlen) 
{
	return 0;
}

static void
intselect_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC register %08x is not implemented\n",address);
        return;
}
uint32_t
intenable_read(void *clientData,uint32_t address,int rqlen) 
{
	return 0;
}

static void
intenable_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC register %08x is not implemented\n",address);
        return;
}
uint32_t
intenclear_read(void *clientData,uint32_t address,int rqlen) 
{
	return 0;
}

static void
intenclear_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC register %08x is not implemented\n",address);
        return;
}
uint32_t
softint_read(void *clientData,uint32_t address,int rqlen) 
{
	return 0;
}

static void
softint_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC register %08x is not implemented\n",address);
        return;
}

uint32_t
softintclear_read(void *clientData,uint32_t address,int rqlen) 
{
	return 0;
}

static void
softintclear_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC register %08x is not implemented\n",address);
        return;
}

uint32_t
protection_read(void *clientData,uint32_t address,int rqlen) 
{
	return 0;
}

static void
protection_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC register %08x is not implemented\n",address);
        return;
}

uint32_t
vectaddr_read(void *clientData,uint32_t address,int rqlen) 
{
	return 0;
}

static void
vectaddr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC register %08x is not implemented\n",address);
        return;
}

uint32_t
defvectaddr_read(void *clientData,uint32_t address,int rqlen) 
{
	return 0;
}

static void
defvectaddr_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC register %08x is not implemented\n",address);
        return;
}

uint32_t
vectaddrn_read(void *clientData,uint32_t address,int rqlen) 
{
	return 0;
}

static void
vectaddrn_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC register %08x is not implemented\n",address);
        return;
}
uint32_t
veccntl_read(void *clientData,uint32_t address,int rqlen) 
{
	return 0;
}

static void
veccntl_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC register %08x is not implemented\n",address);
        return;
}

uint32_t
periphid_read(void *clientData,uint32_t address,int rqlen) 
{
	static uint8_t perid[]={0x90,0x11,0x10,0};
	uint32_t index = (address>>2) & 0x3;
	return perid[index];
}

static void
periphid_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC register %08x is not implemented\n",address);
        return;
}
uint32_t
pcellid_read(void *clientData,uint32_t address,int rqlen) 
{
	static uint8_t cellid[]={0x0d,0xf0,0x05,0xB1};
	uint32_t index = (address>>2) & 0x3;
	return cellid[index];
}

static void
pcellid_write(void *clientData,uint32_t value,uint32_t address,int rqlen) 
{
        fprintf(stderr,"VIC pcellid register at %08x is now writable\n",address);
        return;
}

static void
PL190_Map(void *owner,uint32_t base,uint32_t mask,uint32_t flags)
{
        PL190 *pl=owner;
	int i;
	IOH_New32(base+VICIRQSTATUS,irqstatus_read,irqstatus_write,pl);
	IOH_New32(base+VICFIQSTATUS,fiqstatus_read,fiqstatus_write,pl);
	IOH_New32(base+VICRAWINTR,rawintr_read,rawintr_write,pl);
	IOH_New32(base+VICINTSELECT,intselect_read,intselect_write,pl);
	IOH_New32(base+VICINTENABLE,intenable_read,intenable_write,pl);
	IOH_New32(base+VICINTENCLEAR,intenclear_read,intenclear_write,pl);
	IOH_New32(base+VICSOFTINT,softint_read,softint_write,pl);
	IOH_New32(base+VICSOFTINTCLEAR,softintclear_read,softintclear_write,pl);
	IOH_New32(base+VICPROTECTION,protection_read,protection_write,pl);
	IOH_New32(base+VICVECTADDR,vectaddr_read,vectaddr_write,pl);
	IOH_New32(base+VICDEFVECTADDR,defvectaddr_read,defvectaddr_write,pl);
	for(i=0;i<16;i++) {
		IOH_New32(base+VICVECTADDRN(i),vectaddrn_read,vectaddrn_write,pl);
		IOH_New32(base+VICVECCNTL(i),veccntl_read,veccntl_write,pl);
	}
	for(i=0;i<4;i++) {
		IOH_New32(base+VICPERIPHID(i),periphid_read,periphid_write,pl);
		IOH_New32(base+VICPCELLID(i),pcellid_read,pcellid_write,pl);
	}
}

static void
PL190_UnMap(void *owner,uint32_t base,uint32_t mask)
{
	int i;
	IOH_Delete32(base+VICIRQSTATUS);
	IOH_Delete32(base+VICFIQSTATUS);
	IOH_Delete32(base+VICRAWINTR);
	IOH_Delete32(base+VICINTSELECT);
	IOH_Delete32(base+VICINTENABLE);
	IOH_Delete32(base+VICINTENCLEAR);
	IOH_Delete32(base+VICSOFTINT);
	IOH_Delete32(base+VICSOFTINTCLEAR);
	IOH_Delete32(base+VICPROTECTION);
	IOH_Delete32(base+VICVECTADDR);
	IOH_Delete32(base+VICDEFVECTADDR);
	for(i=0;i<16;i++) {
		IOH_Delete32(base+VICVECTADDRN(i));
		IOH_Delete32(base+VICVECCNTL(i));
	}
	for(i=0;i<4;i++) {
		IOH_Delete32(base+VICPERIPHID(i));
		IOH_Delete32(base+VICPCELLID(i));
	}

}

BusDevice *
PL190_New(char *name,uint32_t base) 
{
	PL190 *pl;
	char *nodename = alloca(strlen(name)+40);
	int i;
	pl = malloc(sizeof(PL190));
	if(!pl) {
		fprintf(stderr,"Out of memory allocating PL190 VIC\n");
		exit(429);
	}
	memset(pl,0,sizeof(PL190));
	for(i=0;i<32;i++) {
		sprintf(nodename,"%s.VICINTSOURCE%d\n",name,i);
		if(!(pl->intSourceNode[i]=SigNode_New(nodename))) {	
			fprintf(stderr,"Can not allocate interrupt node %s\n",nodename);
			exit(324);
		}
		pl->intSourceTrace[i] = SigNode_Trace(pl->intSourceNode[i],int_source_change,pl);
	}
	pl->bdev.first_mapping=NULL;
        pl->bdev.Map=PL190_Map;
        pl->bdev.UnMap=PL190_UnMap;
        pl->bdev.owner=pl;
        pl->bdev.hw_flags=MEM_FLAG_WRITABLE|MEM_FLAG_READABLE;

	return &pl->bdev;
}
