/*
 * ------------------------------------------------------------------------
 * signode.h 
 *      Emulation of Signal nodes 
 *
 * (C) 2004  Lightmaze Solutions AG
 *   Author: Jochen Karrer
 *
 *
 *  This program is free software; you can distribute it and/or modify it
 *  under the terms of the GNU General Public License (Version 2) as
 *  published by the Free Software Foundation.
 *
 *  This program is distributed in the hope it will be useful, but WITHOUT
 *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 *  for more details.
 *
 *  You should have received a copy of the GNU General Public License along
 *  with this program; if not, write to the Free Software Foundation, Inc.,
 *  59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * ------------------------------------------------------------------------
 */
#ifndef SIGNODE_H
#define SIGNODE_H
#include <stdint.h>
#include <stdlib.h>
#include <xy_hash.h>

#define SIG_OPEN      (0)
#define SIG_LOW       (1)
#define SIG_HIGH      (2)
#define SIG_PULLUP    (3)
#define SIG_PULLDOWN  (4)
#define SIG_ILLEGAL   (0x80)

typedef uint64_t SigStamp;

struct SigTrace;
struct SigLink;
typedef struct SigNode {
	char *name;
	XY_HashEntry *hash_entry;
	int selfval;
	int propval;
	SigStamp stamp;
	struct SigLink *linkList;
	struct SigTrace *sigTraceList;
} SigNode;

static inline char * SigName(SigNode *node) {
	return(node->name);
}
typedef struct SigLink {
	struct SigLink * next;
	SigNode *partner;
} SigLink;

typedef int SigTraceProc(struct SigNode *,int value,void *clientData);

typedef struct SigTrace {
	SigTraceProc *proc;
	void *clientData;
	struct SigTrace *next;
	int isactive;
} SigTrace;

void SignodesInit();
SigNode * SigNode_New(char *name); 
void SigNode_Delete(SigNode *signode); 
int SigNode_Set(SigNode *signode,int sigval);
SigTrace * SigNode_Trace(SigNode *node,SigTraceProc *proc,void *clientData);
int SigName_Link(const char *name1,const char *name2);
//void SigName_RemoveLink(const char *name1,const char *name2);
int SigNode_RemoveLink(SigNode *n1, SigNode *n2);
static inline int SigNode_Val(SigNode *signode) { return signode->propval; };
int SigNode_Link(SigNode *sig1, SigNode *sig2); 
#endif
